from magento.client import Magento, make_field_value_query, make_search_query
from magento.version import __version__
from magento.exceptions import MagentoException
from magento.attributes import (
    get_custom_attribute, get_boolean_custom_attribute, get_custom_attributes_dict,
    pretty_custom_attributes
)
from magento.order_helpers import (
    is_order_on_hold, is_order_cash_on_delivery, get_order_shipping_address
)

# ids of the default categories in Magento
ROOT_CATEGORY_ID = 1
DEFAULT_CATEGORY_ID = 2

# Magento visibility options
# https://devdocs.magento.com/guides/v2.4/rest/tutorials/configurable-product/create-configurable-product.html
VISIBILITY_BOTH = 4
VISIBILITY_IN_CATALOG = 2
VISIBILITY_IN_SEARCH = 3
VISIBILITY_NOT_VISIBLE = 1

# Magento product statuses
# https://magento.stackexchange.com/q/10693/92968
ENABLED_PRODUCT = 1
DISABLED_PRODUCT = 2



"""
(Custom) Attributes utilities.
"""
from typing import Callable, Optional, cast, Dict, Any, Union, Sequence, List


def get_custom_attribute(item: dict, attribute_code: str, coerce_as: Optional[Callable] = None):
    """
    Get a custom attribute from an item given its code.

    For example:
        >>> get_custom_attribute(..., "my_custom_attribute")
        "0"

        >>> get_custom_attribute(..., "my_custom_attribute", bool)
        False

    :param item:
    :param attribute_code:
    :param coerce_as: optional callable that is called on the attribute value if it's set.
      This is useful to circumvent Magento's limitation where all attribute values are strings.
    :return: attribute value or None.
    """
    for attribute in item.get("custom_attributes", []):
        if attribute["attribute_code"] == attribute_code:
            value = attribute["value"]
            if coerce_as:
                if coerce_as == bool:
                    # "0" -> False / "1" -> True
                    return bool(int(value))
                return coerce_as(value)
            return value


def get_boolean_custom_attribute(item: dict, attribute_code: str) -> Optional[bool]:
    """
    Equivalent of ``get_custom_attribute(item, attribute_code, coerce_as=bool)`` with proper typing.
    """
    return cast(Optional[bool], get_custom_attribute(item, attribute_code, coerce_as=bool))


def get_custom_attributes_dict(item: Dict[str, Any]) -> Dict[str, Union[Sequence[str], str]]:
    """
    Get all custom attributes from an item as a dict of code->value.
    :param item:
    :return: dict
    """
    d = {}
    for attribute in item.get("custom_attributes", []):
        d[attribute["attribute_code"]] = attribute["value"]

    return d


def pretty_custom_attributes(custom_attributes: List[Dict[str, Any]]):
    """
    Return a human-friendly compact representation of a sequence of custom attributes.

    :param custom_attributes:
    :return:
    """
    pairs = []
    for custom_attribute in custom_attributes:
        k = custom_attribute["attribute_code"]
        v = custom_attribute["value"]
        pairs.append(f"{k}={repr(v)}")

    return ", ".join(pairs)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['magento']

package_data = \
{'': ['*']}

install_requires = \
['api-session>=1.2.0,<2.0.0']

extras_require = \
{'docs': ['Sphinx>=4.4.0,<5.0.0', 'sphinx-rtd-theme>=1.0.0,<2.0.0']}

setup_kwargs = {
    'name': 'pymagento',
    'version': '1.3.1',
    'description': 'Python client for the Magento 2 API',
    'long_description': '# PyMagento\n\n**PyMagento** is a Python client for the Magento 2 API. Its goal is to provide an easy-to-use\nPythonic interface to the Magento 2 API, while being lightweight and extendable.\n\n* [Read the docs](https://pymagento2.readthedocs.io/)\n\n\nNote: PyMagento is not affiliated to nor endorsed by Adobe or the Magento team.\n\n## Install\n\nWe support only Python 3.8+.\n\n### Pip\n\n    python -m pip install pymagento\n\n### Poetry\n\n    poetry add pymagento\n\n## Usage\n\n```python\nimport magento\n\nclient = magento.Magento(base_url="...", token="...", scope="all")\n\nproduct = client.get_product("SKU123")\nprint(magento.get_custom_attribute(product, "description"))\n```\n\nFor more information, [read the docs](https://pymagento2.readthedocs.io/).\n\n## License\n\nCopyright 2020-2022 [Bixoto](https://bixoto.com/).\n',
    'author': 'Bixoto',
    'author_email': 'info@bixoto.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Bixoto/PyMagento',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
